@@ -1,6 +1,6 @@
-/* $Id: rmdraw.c,v 1.22 2005/03/05 16:18:39 rmagick Exp $ */
+/* $Id: rmdraw.c,v 1.43.2.2.2.2 2008/02/24 23:25:07 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2005 by Timothy P. Hunter
+|                Copyright (C) 2008 by Timothy P. Hunter
 | Name:     rmdraw.c
 | Author:   Tim Hunter
 | Purpose:  Contains Draw class methods.
@@ -13,9 +13,10 @@
 static void mark_Draw(void *);
 static void destroy_Draw(void *);
 static void destroy_Montage(void *);
+static VALUE new_DrawOptions(void);
 
 typedef MagickBooleanType (get_type_metrics_func_t)(Image *, const DrawInfo *, TypeMetric *);
-static VALUE get_type_metrics(int, VALUE *, VALUE, get_type_metrics_func_t *);
+static VALUE get_type_metrics(int, VALUE *, VALUE, get_type_metrics_func_t);
 
 
 /*
@@ -354,7 +355,37 @@ Draw_text_antialias_eq(VALUE self, VALUE text_antialias)
 
     rm_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
-    draw->info->text_antialias = RTEST(text_antialias);
+    draw->info->text_antialias = (MagickBooleanType) RTEST(text_antialias);
+    return self;
+}
+
+/*
+    Method:     Draw#tile=
+    Purpose:    tile attribute writer
+*/
+VALUE
+Draw_tile_eq(VALUE self, VALUE image)
+{
+    Draw *draw;
+    Image *tile;
+
+    rm_check_frozen(self);
+    Data_Get_Struct(self, Draw, draw);
+
+    if (draw->info->fill_pattern)
+    {
+        (void) DestroyImage(draw->info->fill_pattern);
+        draw->info->fill_pattern = NULL;
+    }
+
+    if (image == Qnil)
+    {
+        return self;
+    }
+
+    Data_Get_Struct(ImageList_cur_image(image), Image, tile);
+    draw->info->fill_pattern = rm_clone_image(tile);
+
     return self;
 }
 
@@ -412,8 +443,16 @@ VALUE Draw_annotate(
     rm_check_frozen(ImageList_cur_image(image_arg));
     Data_Get_Struct(ImageList_cur_image(image_arg), Image, image);
 
-    // Copy text to Draw structure
-    magick_clone_string(&draw->info->text, STRING_PTR(text));
+    // Translate & store in Draw structure
+#if defined(HAVE_INTERPRETIMAGEATTRIBUTES)
+    draw->info->text = InterpretImageAttributes(NULL, image, STRING_PTR(text));
+#else
+    draw->info->text = TranslateText(NULL, image, STRING_PTR(text));
+#endif
+    if (!draw->info->text)
+    {
+           rb_raise(rb_eArgError, "no text");
+    }
 
     // Create geometry string, copy to Draw structure, overriding
     // any previously existing value.
@@ -422,13 +461,8 @@ VALUE Draw_annotate(
     x      = NUM2LONG(x_arg);
     y      = NUM2LONG(y_arg);
 
-    // If either the width or height is 0, both must be 0.
-    if (width == 0 || height == 0)
+    if (width == 0 && height == 0)
     {
-        if (width != 0 || height != 0)
-        {
-            rb_raise(rb_eArgError, "invalid geometry %lux%lu%+ld%+ld", width, height, x, y);
-        }
         sprintf(geometry_str, "%+ld%+ld", x, y);
     }
 
@@ -442,9 +476,11 @@ VALUE Draw_annotate(
 
     (void) AnnotateImage(image, draw->info);
 
+    magick_free(draw->info->text);
+    draw->info->text = NULL;
     draw->info->affine = keep;
 
-    HANDLE_ERROR_IMG(image)
+    rm_check_image_exception(image, RetainOnError);
 
     return self;
 }
@@ -617,7 +653,7 @@ Draw_draw(VALUE self, VALUE image_arg)
     magick_clone_string(&(draw->info->primitive), STRING_PTR(draw->primitives));
 
     (void) DrawImage(image, draw->info);
-    HANDLE_ERROR_IMG(image)
+    rm_check_image_exception(image, RetainOnError);
 
     magick_free(draw->info->primitive);
     draw->info->primitive = NULL;
@@ -644,7 +680,7 @@ Draw_dup(VALUE self)
     {
         (void)rb_obj_taint(dup);
     }
-    return rb_funcall(dup, ID_initialize_copy, 1, self);
+    return rb_funcall(dup, rm_ID_initialize_copy, 1, self);
 }
 
 
@@ -705,42 +741,23 @@ VALUE Draw_init_copy(VALUE self, VALUE orig)
     return self;
 }
 
+
 /*
     Method:     Draw#initialize <{ info initializers }>
     Purpose:    Initialize Draw object
-    Notes:      Here are the DrawInfo fields that are copied from Info.
-                These are the only Info fields that can be usefully
-                set in the initializer block.
-
-                DrawInfo            Info
-                --------            ---------
-                stroke_antialias    antialias
-                font                font
-                density             density
-                text_antialias      antialias
-                pointsize           pointsize
-                border_color        border_color
-                server_name         server_name
-                debug               debug
 */
 VALUE
 Draw_initialize(VALUE self)
 {
-    Draw *draw;
-    Info *info;
-    volatile VALUE info_obj;
+    Draw *draw, *draw_options;
+    volatile VALUE options;
 
     Data_Get_Struct(self, Draw, draw);
 
-    // Create a new Info object, running the info parms block in the process
-    info_obj = rm_info_new();
-
-    // Use the Info structure to create the DrawInfo structure
-    Data_Get_Struct(info_obj, Info, info);
-    draw->info = CloneDrawInfo(info, NULL);
-
-    draw->primitives = (VALUE)0;
-    draw->tmpfile_ary = NULL;
+    options = new_DrawOptions();
+    Data_Get_Struct(options, Draw, draw_options);
+    draw->info = draw_options->info;
+    draw_options->info = NULL;
 
     return self;
 }
@@ -763,11 +780,11 @@ Draw_inspect(VALUE self)
     Purpose:    Create a new Draw object
     Raises:     ImageMagickError if no memory
 */
-VALUE
+
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-Draw_alloc(VALUE class)
+VALUE Draw_alloc(VALUE class)
 #else
-Draw_new(VALUE class)
+VALUE Draw_new(VALUE class)
 #endif
 {
     Draw *draw;
@@ -835,7 +852,11 @@ destroy_Draw(void *drawptr)
     Draw *draw = (Draw *)drawptr;
     struct TmpFile_Name *tmpfile;
 
-    DestroyDrawInfo(draw->info);
+    if (draw->info)
+    {
+        (void) DestroyDrawInfo(draw->info);
+        draw->info = NULL;
+    }
 
     // Erase any temporary image files.
     while (draw->tmpfile_ary)
@@ -849,6 +870,76 @@ destroy_Draw(void *drawptr)
     xfree(drawptr);
 }
 
+
+/*
+    Static:     new_DrawOptions
+    Purpose:    Allocate & initialize a DrawOptions object.
+*/
+static VALUE new_DrawOptions(void)
+{
+#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+    return DrawOptions_initialize(Draw_alloc(Class_DrawOptions));
+#else
+    return DrawOptions_new(Class_DrawOptions);
+#endif
+}
+
+
+/*
+    Method:     DrawOptions#allocate
+                DarwOptions#new
+    Purpose:    Create a DrawOptions object
+    Notes:      The DrawOptions class is the same as the Draw class except
+                is has only the attribute writer functions.
+*/
+#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+VALUE DrawOptions_alloc(VALUE class)
+#else
+VALUE DrawOptions_new(VALUE class)
+#endif
+{
+    Draw *draw_options;
+    volatile VALUE draw_options_obj;
+
+    draw_options = ALLOC(Draw);
+    memset(draw_options, 0, sizeof(Draw));
+    draw_options_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw_options);
+
+#if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+    rb_obj_call_init(draw_options_obj, 0, NULL);
+#endif
+
+    return draw_options_obj;
+}
+
+
+/*
+    Method:     DrawOptions#initialize
+    Purpose:    Initialize a DrawOptions object
+*/
+VALUE DrawOptions_initialize(VALUE self)
+{
+    Draw *draw_options;
+
+    Data_Get_Struct(self, Draw, draw_options);
+    draw_options->info = magick_malloc(sizeof(DrawInfo));
+    if (!draw_options->info)
+    {
+        rb_raise(rb_eNoMemError, "not enough memory to continue");
+    }
+
+    GetDrawInfo(NULL, draw_options->info);
+
+    if (rb_block_given_p())
+    {
+        // Run the block in self's context
+        (void) rb_obj_instance_eval(0, NULL, self);
+    }
+
+    return self;
+}
+
+
 /*
     Method:     Magick::Montage#background_color(color-name)
     Purpose:    set background_color value
@@ -962,7 +1053,7 @@ Montage_frame_eq(VALUE self, VALUE frame_arg)
     volatile VALUE frame;
 
     Data_Get_Struct(self, Montage, montage);
-    frame = rb_funcall(frame_arg, ID_to_s, 0);
+    frame = rb_funcall(frame_arg, rm_ID_to_s, 0);
     magick_clone_string(&montage->info->frame, STRING_PTR(frame));
 
     return self;
@@ -979,7 +1070,7 @@ Montage_geometry_eq(VALUE self, VALUE geometry_arg)
     volatile VALUE geometry;
 
     Data_Get_Struct(self, Montage, montage);
-    geometry = rb_funcall(geometry_arg, ID_to_s, 0);
+    geometry = rb_funcall(geometry_arg, rm_ID_to_s, 0);
     magick_clone_string(&montage->info->geometry, STRING_PTR(geometry));
 
     return self;
@@ -1048,7 +1139,7 @@ Montage_new(VALUE class)
     }
 
     montage_info = CloneMontageInfo(image_info, NULL);
-    DestroyImageInfo(image_info);
+    (void) (void) DestroyImageInfo(image_info);
 
     if (!montage_info)
     {
@@ -1061,7 +1152,7 @@ Montage_new(VALUE class)
     montage_obj = Data_Wrap_Struct(class, NULL, destroy_Montage, montage);
 
 #if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-    rb_obj_call_init(montage_obj, 0, NULL);
+    (void) rb_obj_call_init(montage_obj, 0, NULL);
 #endif
 
     return montage_obj;
@@ -1093,11 +1184,16 @@ destroy_Montage(void *obj)
     Montage *montage = obj;
 
     // If we saved a temporary texture image, delete it now.
-    if (montage->info->texture != NULL)
+    if (montage->info && montage->info->texture != NULL)
     {
         rm_delete_temp_image(montage->info->texture);
+        montage->info->texture = NULL;
+    }
+    if (montage->info)
+    {
+        (void) DestroyMontageInfo(montage->info);
+        montage->info = NULL;
     }
-    DestroyMontageInfo(montage->info);
     xfree(montage);
 }
 
@@ -1125,7 +1221,7 @@ Montage_shadow_eq(VALUE self, VALUE shadow)
     Montage *montage;
 
     Data_Get_Struct(self, Montage, montage);
-    montage->info->shadow = RTEST(shadow);
+    montage->info->shadow = (MagickBooleanType) RTEST(shadow);
     return self;
 }
 
@@ -1184,7 +1280,7 @@ Montage_tile_eq(VALUE self, VALUE tile_arg)
     volatile VALUE tile;
 
     Data_Get_Struct(self, Montage, montage);
-    tile = rb_funcall(tile_arg, ID_to_s, 0);
+    tile = rb_funcall(tile_arg, rm_ID_to_s, 0);
     magick_clone_string(&montage->info->tile, STRING_PTR(tile));
 
     return self;
@@ -1205,13 +1301,114 @@ Montage_title_eq(VALUE self, VALUE title)
 }
 
 
+/*
+    Extern:     PolaroidOptions_alloc()
+    Purpose:    Allocate a new Magick::PolaroidOptions object
+    Notes:      Internally a PolaroidOptions object is the same as a Draw
+                object. The methods are implemented by Draw methods in
+                rmdraw.c.
+*/
+VALUE
+PolaroidOptions_alloc(VALUE class)
+{
+    volatile VALUE polaroid_obj;
+    ImageInfo *image_info;
+    Draw *draw;
+
+    image_info = CloneImageInfo(NULL);
+
+    draw = ALLOC(Draw);
+    memset(draw, 0, sizeof(*draw));
+
+    draw->info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+    (void)(void) DestroyImageInfo(image_info);
+
+    polaroid_obj = Data_Wrap_Struct(class, NULL, destroy_Draw, draw);
+
+    return polaroid_obj;
+}
+
+
+/*
+    Method:     PolaroidOptions.new
+    Purpose:    Ruby 1.6 singleton function to allocate and init a PolaroidOptions
+                object
+*/
+#if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+VALUE PolaroidOptions_new(VALUE class)
+{
+    volatile VALUE polaroid_obj;
+
+    polaroid_obj = PolaroidOptions_alloc(class);
+    rb_obj_call_init(polaroid_obj, 0, NULL);
+
+    return polaroid_obj;
+}
+#endif
+
+
+/*
+    Method:     Magick::PolaroidOptions#initialize
+    Purpose:    Yield to an optional block
+*/
+VALUE
+PolaroidOptions_initialize(VALUE self)
+{
+    Draw *draw;
+    ExceptionInfo exception;
+
+    // Default shadow color
+    Data_Get_Struct(self, Draw, draw);
+
+    GetExceptionInfo(&exception);
+    (void) QueryColorDatabase("gray75", &draw->shadow_color, &exception);
+    CHECK_EXCEPTION()
+
+    if (rb_block_given_p())
+    {
+        // Run the block in self's context
+        (void) rb_obj_instance_eval(0, NULL, self);
+    }
+    return self;
+}
+
+
+/*
+    Extern:     rm_polaroid_new
+    Purpose:    allocate a PolaroidOptions instance
+    Notes:      Internal use
+*/
+VALUE rm_polaroid_new(void)
+{
+#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+    return PolaroidOptions_initialize(PolaroidOptions_alloc(Class_PolaroidOptions));
+#else
+    return PolaroidOptions_new(Class_PolaroidOptions);
+#endif
+}
+
+
+/*
+    Method:     PolaroidOptions#shadow_color=
+    Purpose:    Set the shadow color attribute
+*/
+VALUE PolaroidOptions_shadow_color_eq(VALUE self, VALUE shadow)
+{
+    Draw *draw;
+
+    Data_Get_Struct(self, Draw, draw);
+    Color_to_PixelPacket(&draw->shadow_color, shadow);
+    return self;
+}
+
+
 static VALUE get_dummy_tm_img(VALUE klass)
 {
     volatile VALUE dummy_img = 0;
     Info *info;
     Image *image;
 
-    if (rb_cvar_defined(klass, ID__dummy_img_) != Qtrue)
+    if (rb_cvar_defined(klass, rm_ID__dummy_img_) != Qtrue)
     {
 
         info = CloneImageInfo(NULL);
@@ -1224,13 +1421,13 @@ static VALUE get_dummy_tm_img(VALUE klass)
         {
             rb_raise(rb_eNoMemError, "not enough memory to continue");
         }
-        DestroyImageInfo(info);
+        (void) DestroyImageInfo(info);
         dummy_img = rm_image_new(image);
 
-        RUBY18(rb_cvar_set(klass, ID__dummy_img_, dummy_img, 0));
-        RUBY16(rb_cvar_set(klass, ID__dummy_img_, dummy_img));
+        RUBY18(rb_cvar_set(klass, rm_ID__dummy_img_, dummy_img, 0));
+        RUBY16(rb_cvar_set(klass, rm_ID__dummy_img_, dummy_img));
     }
-    dummy_img = rb_cvar_get(klass, ID__dummy_img_);
+    dummy_img = rb_cvar_get(klass, rm_ID__dummy_img_);
 
     return dummy_img;
 }
@@ -1246,13 +1443,14 @@ get_type_metrics(
      int argc,
      VALUE *argv,
      VALUE self,
-     get_type_metrics_func_t *getter)
+     get_type_metrics_func_t getter)
 {
-     static char attrs[] = "bcdefghiklmnopqrstuwxyz";
- #define ATTRS_L (sizeof(attrs)-1)
+     static char attrs[] = "OPbcdefghiklmnopqrstuwxyz[@#%";
+ #define ATTRS_L ((int)(sizeof(attrs)-1))
      Image *image;
      Draw *draw;
      TypeMetric metrics;
+     volatile VALUE img;
      char *text = NULL;
      long text_l;
      long x;
@@ -1283,26 +1481,47 @@ get_type_metrics(
                  }
              }
 
-             Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
+             img = get_dummy_tm_img(CLASS_OF(self));
+             Data_Get_Struct(img, Image, image);
              break;
          case 2:
-             Data_Get_Struct(ImageList_cur_image(argv[0]), Image, image);
-             text = STRING_PTR(argv[1]);
+             img = ImageList_cur_image(argv[0]);
+             Data_Get_Struct(img, Image, image);
+             text = STRING_PTR_LEN(argv[1], text_l);
              break;                  // okay
          default:
              rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
              break;
      }
 
+     if (text_l == 0)
+     {
+         rb_raise(rb_eArgError, "no text to measure");
+     }
+
      Data_Get_Struct(self, Draw, draw);
-     magick_clone_string(&draw->info->text, text);
+#if defined(HAVE_INTERPRETIMAGEATTRIBUTES)
+     draw->info->text = InterpretImageAttributes(NULL, image, text);
+#else
+     draw->info->text = TranslateText(NULL, image, text);
+#endif
+     if (!draw->info->text)
+     {
+         rb_raise(rb_eArgError, "no text to measure");
+     }
 
      okay = (*getter)(image, draw->info, &metrics);
 
+     magick_free(draw->info->text);
+     draw->info->text = NULL;
+
      if (!okay)
      {
-         rb_warning("RMagick: get_type_metrics failed");
-         return Qnil;
+         rm_check_image_exception(image, RetainOnError);
+
+         // Shouldn't get here...
+         rb_raise(rb_eRuntimeError, "Can't measure text. Are the fonts installed? "
+                  "Is the FreeType library installed?");
      }
      return TypeMetric_from_TypeMetric(&metrics);
 }
