@@ -1,6 +1,6 @@
-/* $Id: rmilist.c,v 1.46.2.1 2007/03/04 00:05:34 rmagick Exp $ */
+/* $Id: rmilist.c,v 1.46.2.1.2.3 2008/09/10 23:25:13 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2007 by Timothy P. Hunter
+|                Copyright (C) 2008 by Timothy P. Hunter
 | Name:     rmilist.c
 | Author:   Tim Hunter
 | Purpose:  ImageList class method definitions for RMagick
@@ -491,7 +491,7 @@ ImageList_optimize_layers(VALUE self, VALUE method)
     Notes:      this simply calls ImageList.new() in RMagick.rb
 */
 VALUE
-rm_imagelist_new()
+rm_imagelist_new(void)
 {
     return rb_funcall(Class_ImageList, rm_ID_new, 0);
 }
@@ -653,7 +653,7 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     // Create new ImageList object, convert mapped image sequence to images,
     // append to images array.
     new_imagelist = rm_imagelist_new();
-    while ((new_image = ShiftImageList(&new_images)))
+    while ((new_image = RemoveFirstImageFromList(&new_images)))
     {
         rm_imagelist_push(new_imagelist, rm_image_new(new_image));
     }
@@ -710,9 +710,11 @@ ImageList_to_blob(VALUE self)
 #else
     blob = ImageToBlob(info, images, &length, &exception);
 #endif
-    if (exception.severity != UndefinedException)
+    if (blob && exception.severity >= ErrorException)
     {
         magick_free((void*)blob);
+        blob = NULL;
+        length = 0;
     }
     rm_split(images);
     CHECK_EXCEPTION()
